package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.common.module.entity.Project;
import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.service.IProjectService;
import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.kuaishou.modules.ai.entity.KuaishouProjectStrategy;
import cn.com.ctop.kuaishou.modules.ai.service.IKuaishouProjectStrategyService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.DateUtils;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 自动投放-项目策略表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-06-24
 */
@Slf4j
@Api(tags = "自动投放-项目策略表")
@RestController
@RequestMapping("/kuaishouProjectStrategy")
public class KuaishouProjectStrategyController {
    @Autowired
    private IKuaishouProjectStrategyService kuaishouProjectStrategyService;
    @Autowired
    private IUserAllocationService userAllocationService;
    @Autowired
    private IProjectService projectService;


    /**
     * 项目配置列表
     */
    @PostMapping(value = "/queryProjectList")
    public Result<Object> queryProjectList(@RequestBody JSONObject data) {
        try {
            if (Check.isNull(data.getString("startTime")) || Check.isNull(data.getString("endTime"))) {
                return Result.error("请选择查询时间");
            }
            return kuaishouProjectStrategyService.queryProjectList(data);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 项目策略中账户列表
     */
    @PostMapping(value = "/queryAccountList")
    public Result<Object> queryAccountList(@RequestBody JSONObject data) {
        try {
            if (Check.isNull(data.getString("startTime")) || Check.isNull(data.getString("endTime"))) {
                return Result.error("请选择查询时间");
            }
            return kuaishouProjectStrategyService.queryAccountList(data);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 修改项目策略状态
     */
    @PostMapping(value = "/updateProjectStrategy")
    public Result<Object> updateProjectStrategy(@RequestBody KuaishouProjectStrategy kuaishouProjectStrategy) {
        KuaishouProjectStrategy kuaishouProjectStrategyEntity = kuaishouProjectStrategyService.getById(kuaishouProjectStrategy.getId());
        if (kuaishouProjectStrategyEntity == null) {
            return Result.error("未找到对应实体");
        } else {
            boolean ok = kuaishouProjectStrategyService.updateById(kuaishouProjectStrategy);
            if (ok) {
                return Result.ok("修改成功!");
            }
        }
        return Result.ok("修改失败!");
    }

    /**
     * 修改项目策略状态
     */
    @PostMapping(value = "/updateAccountStatus")
    public Result<Object> updateAccountStatus(@RequestBody JSONObject data) {
        try {
            Long accountId = data.getLong("accountId");
            Integer projectStrategy = data.getInteger("projectStrategy");
            if (Check.isNull(accountId) || Check.isNull(projectStrategy)) {
                return Result.error("缺失参数");
            }
            boolean ok = userAllocationService.updateProjectStrategyStatus(accountId, projectStrategy);
            if (ok) {
                return Result.ok("修改成功!");
            }
        } catch (Exception e) {
            log.error("修改异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("修改失败");
    }

    /**
     * 校验是否有开启的项目
     */
    @GetMapping(value = "/checkProjectisOpen")
    public Result<Object> checkProjectisOpen(Long projectId) {
        try {
            if (Check.isNull(projectId)) {
                return Result.error("缺失参数");
            }
            boolean ok = kuaishouProjectStrategyService.checkProjectisOpen(projectId);
            if (ok) {
                return Result.ok((Object) 1);
            }
        } catch (Exception e) {
            log.error("修改异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.ok((Object) 0);
    }

    /**
     * 出价反显展示
     */
    @GetMapping(value = "/queryBidInfo")
    public Result<Object> queryBidInfo(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouProjectStrategy> result = new Result<>();
        KuaishouProjectStrategy kuaishouProjectStrategy = kuaishouProjectStrategyService.getById(id);
        if (kuaishouProjectStrategy == null) {
            return Result.error("未找到对应实体");
        } else {
            return Result.ok(kuaishouProjectStrategy);
        }
    }

    /**
     * 查询项目策略匹配结果
     */
    @GetMapping(value = "/queryMatchResult")
    public Result<Object> queryMatchResult(Long projectId, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo, @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        Result<KuaishouProjectStrategy> result = new Result<>();
        try {
            return kuaishouProjectStrategyService.queryMatchResult(projectId, pageNo, pageSize);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }


    /**
     * 创建项目策略
     */
    @PostMapping(value = "/createProjectStrategy")
    public Result<KuaishouProjectStrategy> createProjectStrategy(@RequestBody JSONObject data) {
        Result<KuaishouProjectStrategy> result = new Result<>();
        try {
            KuaishouProjectStrategy kuaishouProjectStrategy = JSONObject.parseObject(data.toJSONString(), KuaishouProjectStrategy.class);
            kuaishouProjectStrategy.setStatDate(DateUtils.getNowDate("yyyy-MM-dd"));
            LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
            if (!Check.isNull(user)) {
                kuaishouProjectStrategy.setUserId(user.getId());
            }
            if (Check.isNull(kuaishouProjectStrategy.getId()) && !Check.isNull(kuaishouProjectStrategy.getProjectId())) {
                QueryWrapper<UserAllocation> wrapper = new QueryWrapper<>();
                wrapper.eq("project_id", kuaishouProjectStrategy.getProjectId());
                wrapper.eq("account_status", 0);
                wrapper.eq("auto_delivery", 0);
                UserAllocation userAllocation = new UserAllocation();
                userAllocation.setAutoDelivery(1);
                userAllocationService.update(userAllocation, wrapper);
            }
            boolean b = kuaishouProjectStrategyService.saveOrUpdate(kuaishouProjectStrategy);
            result.success("操作成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }


    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "自动投放-项目策略表-通过id删除", notes = "自动投放-项目策略表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            kuaishouProjectStrategyService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "自动投放-项目策略表-批量删除", notes = "自动投放-项目策略表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouProjectStrategy> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<KuaishouProjectStrategy> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouProjectStrategyService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "自动投放-项目策略表-通过id查询", notes = "自动投放-项目策略表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouProjectStrategy> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouProjectStrategy> result = new Result<>();
        KuaishouProjectStrategy kuaishouProjectStrategy = kuaishouProjectStrategyService.getById(id);
        if (kuaishouProjectStrategy == null) {
            result.error500("未找到对应实体");
        } else {
            Project project = projectService.getById(kuaishouProjectStrategy.getProjectId());
            if (!Check.isNull(project)) {
                kuaishouProjectStrategy.setProjectName(project.getProjectName());
            }
            result.setResult(kuaishouProjectStrategy);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouProjectStrategy> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouProjectStrategy kuaishouProjectStrategy = JSON.parseObject(deString, KuaishouProjectStrategy.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouProjectStrategy, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouProjectStrategy> pageList = kuaishouProjectStrategyService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "自动投放-项目策略表列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouProjectStrategy.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("自动投放-项目策略表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouProjectStrategy> listKuaishouProjectStrategys = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouProjectStrategy.class, params);
                kuaishouProjectStrategyService.saveBatch(listKuaishouProjectStrategys);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouProjectStrategys.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
